#include "DirectX2dPainter.h"

#include "DirectX2dInterface.h"
#include "AbstractGeometricItem.h"
#include "Rectangle.h"
#include "SceneModel.h"

void DirectX2dPainter::startDrawing()
{
	mD2dInterface->getRenderTarget()->BeginDraw();
}

void DirectX2dPainter::finishDrawing()
{
	mD2dInterface->getRenderTarget()->EndDraw();
}

D2D1::ColorF DirectX2dPainter::toD2dColor(const Color& color)
{
	return D2D1::ColorF(static_cast<float>(color.getR() / 255.0), static_cast<float>(color.getG() / 255.0), 
		static_cast<float>(color.getB() / 255.0), static_cast<float>(color.getAlpha()));
}

void DirectX2dPainter::clearBackground(const Color& color)
{
	mD2dInterface->getRenderTarget()->Clear(toD2dColor(color));
}

void DirectX2dPainter::paint(SceneModel* model)
{
	auto rt = mD2dInterface->getRenderTarget();

	if (model->getGeometry()->getType() == AbstractGeometricItem::Type::RECTANGLE)
	{
		const auto loc = model->getTransform().getLocation();
		const auto scale_x = model->getTransform().getScaleX();
		const auto scale_y = model->getTransform().getScaleY();
		D2D1_RECT_F d2d_rect{ static_cast<float>(loc.getX()), static_cast<float>(loc.getY() + scale_x), static_cast<float>(loc.getX() + scale_y), static_cast<float>(loc.getY()) };

		if (model->hasFillColor())
		{
			mSolidBrush->SetColor(toD2dColor(model->getFillColor()));
			rt->FillRectangle(d2d_rect, mSolidBrush.Get());
		}

		if (model->hasOutlineColor())
		{
			mSolidBrush->SetColor(toD2dColor(model->getOutlineColor()));
			rt->DrawRectangle(d2d_rect, mSolidBrush.Get(), 1.0f);
		}	
	}
}

void DirectX2dPainter::setD2dInterface(DirectX2dInterface* d2dIterface)
{
	mD2dInterface = d2dIterface;
	auto hr = mD2dInterface->getRenderTarget()->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black, 1.0f), &mSolidBrush);
}